@@ -1,8 +1,8 @@
 /* gnome-stones - main.c
  *
- * Time-stamp: <2002/06/03 16:02:27 ross>
+ * Time-stamp: <2003-06-03 22:02:25 callum>
  *
- * Copyright (C) 1998 Carsten Schaar
+ * Copyright (C) 1998, 2003 Carsten Schaar
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -161,7 +161,7 @@ load_image_from_path (const char *relative_name)
       widget= gtk_message_dialog_new (GTK_WINDOW(app), 
                                       GTK_DIALOG_DESTROY_WITH_PARENT,
                                       GTK_MESSAGE_ERROR, 
-                                      GTK_BUTTONS_CLOSE, 
+                                      GTK_BUTTONS_OK, 
                                       buffer);
       gtk_dialog_run(GTK_DIALOG(widget));
 
@@ -429,8 +429,6 @@ game_widget_key_release_callback (GtkWidget   *widget,
 {
   if (last_keyval == event->keyval)
     {
-      player_x_direction=  0;
-      player_y_direction=  0;
       last_keyval       =  GDK_VoidSymbol;
       return TRUE;
     }
@@ -808,10 +806,8 @@ iteration_timeout_function (gpointer data)
   
   cave_iterate (cave, player_x_direction, player_y_direction, player_push);
 
-#ifndef USE_KEY_RELEASE
   player_x_direction= 0;
   player_y_direction= 0;
-#endif 
   
   if (!(flags & CAVE_FINISHED) && (cave->flags & CAVE_FINISHED))
     {
@@ -1010,6 +1006,11 @@ game_pause_cb (GtkWidget *widget, gpointer data)
     }
 }
 
+static void
+level_restart_cb (GtkWidget * widget, gpointer date)
+{
+  cave_player_die (cave);
+}
 
 static void
 preferences_cb (GtkWidget *widget, gpointer data)
@@ -1042,7 +1043,8 @@ about_cb (GtkWidget *widget, gpointer data)
   GdkPixbuf *pixbuf = NULL;
   
   const gchar *authors[]= {
-    "Carsten Schaar <nhadcasc@fs-maphy.uni-hannover.de>",
+    N_("Main program: Carsten Schaar <nhadcasc@fs-maphy.uni-hannover.de>"),
+    N_("Sokoban levels: Herman Mansilla"),
     NULL
   };
   gchar *documenters[] = {
@@ -1092,6 +1094,7 @@ about_cb (GtkWidget *widget, gpointer data)
 static GnomeUIInfo game_menu[]= {
   GNOMEUIINFO_MENU_NEW_GAME_ITEM(game_start_cb, NULL),
   GNOMEUIINFO_MENU_END_GAME_ITEM(game_stop_cb, NULL),
+  GNOMEUIINFO_ITEM_STOCK(N_("_Restart level"), N_("Restart the current level"), level_restart_cb, GTK_STOCK_REFRESH),
   GNOMEUIINFO_SEPARATOR,
   GNOMEUIINFO_MENU_PAUSE_GAME_ITEM (game_pause_cb, NULL),
   GNOMEUIINFO_SEPARATOR,
@@ -1336,8 +1339,6 @@ main (int argc, char *argv[])
 
   joystick_set_widget (app);
 
-  gtk_widget_realize (app);
-
   gtk_widget_show (app);
 
   title_image_load ();
@@ -1356,12 +1357,3 @@ main (int argc, char *argv[])
 
   return 0;
 }
-
-
-/* Local Variables: */
-/* mode:c */
-/* eval:(load-library "time-stamp") */
-/* eval:(make-local-variable 'write-file-hooks) */
-/* eval:(add-hook 'write-file-hooks 'time-stamp) */
-/* eval:(setq time-stamp-format '(time-stamp-yyyy/mm/dd time-stamp-hh:mm:ss user-login-name)) */
-/* End: */
