@@ -1,8 +1,8 @@
 /* gnome-stones - preferences.h
  *
- * Time-stamp: <1999/03/03 18:17:52 carsten>
+ * Time-stamp: <2003/05/29 20:01:36 callum>
  *
- * Copyright (C) 1998 Carsten Schaar
+ * Copyright (C) 1998, 2003 Carsten Schaar
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "preferences.h"
+#include "status.h"
+#include "main.h"
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <dirent.h>
@@ -28,6 +35,15 @@
 /*****************************************************************************/
 /* Global Variables */
 
+/* Enum for preferences dialog */
+enum {
+     GAME_STRING,
+     CAVES_INT,
+     FILENAME_STRING,
+     INDEX_INT,
+     N_COLS
+};
+
 /* The default game, that should be loaded, if no client state is to
    be restored.  If this variables value is NULL, than 'default.cave'
    will be used instead.  */
@@ -63,6 +79,12 @@ guint start_cave= 0;
 #endif
 gfloat   joystick_switch_level= 0.5;
 
+
+/* This is default scroll method for view function */
+
+void (*view_scroll_method) (GStonesView *view, GStonesCave *cave)= atari_scroll;
+
+
 /* The game can be in different states.  These state decides, how to
    react if some events occur.  */
 
@@ -142,6 +164,15 @@ add_game (const char *filename)
 }
 
 
+gboolean caves_suffix (char *name)
+{
+ int l=strlen (name); 
+ if (l<7) return FALSE;
+
+ return strcmp (name+l-6, ".caves")==0;
+
+}
+
 void
 game_directory_scan (const char *directory)
 {
@@ -157,7 +188,8 @@ game_directory_scan (const char *directory)
 	  char *filename= g_strconcat (directory, "/", entry->d_name, NULL);
 	  struct stat sbuf;
 	  
-	  if ((stat (filename, &sbuf)== 0) && S_ISREG (sbuf.st_mode))
+	  if ( caves_suffix (filename) 
+	       && (stat (filename, &sbuf)== 0) && S_ISREG (sbuf.st_mode))
 	    add_game (filename);
 
 	  g_free (filename);
@@ -195,6 +227,35 @@ load_game_by_name (const char *filename, guint cave)
 }
 
 
+
+/*****************************************************************************/
+/* string<-->scroll_method conversion  */
+
+
+gchar *scroll_method_name (void)
+{
+  if (view_scroll_method==atari_scroll)
+    return "atari_scroll";
+  if (view_scroll_method==smooth_scroll)
+    return "smooth_scroll";
+  if (view_scroll_method==center_scroll)
+    return "center_scroll";
+ 
+  return "default";
+
+}
+
+void set_scroll_method (gchar *name)
+{
+
+  if (!strcmp (name,"atari_scroll"))  view_scroll_method=atari_scroll;
+  if (!strcmp (name,"smooth_scroll")) view_scroll_method=smooth_scroll;
+  if (!strcmp (name,"center_scroll")) view_scroll_method=center_scroll;
+
+}
+
+
+
 
 /*****************************************************************************/
 /* Save preferences.  */
@@ -221,9 +282,11 @@ preferences_save (gboolean global)
 #endif  
   if (devicename)
     gnome_config_set_string ("Preferences/Joystick device",  devicename);
-  gnome_config_set_float ("Preferences/Joytick switch level", 
+  gnome_config_set_float ("Preferences/Joystick switch level", 
 			  joystick_switch_level);
 
+  gnome_config_set_string ("Preferences/Scroll method", scroll_method_name());
+
   if (game)
     {
       gnome_config_set_string ("Preferences/Game", game->filename);
@@ -321,7 +384,10 @@ preferences_restore (void)
 #endif
 
   joystick_switch_level= 
-    gnome_config_get_float ("Preferences/Joytick switch level=0.5");
+    gnome_config_get_float ("Preferences/Joystick switch level=0.5");
+
+  set_scroll_method ( 
+    gnome_config_get_string ("Preferences/Scroll method=atari_scroll") );
 
 
   filename= gnome_config_get_string_with_default ("Preferences/Game", &def);
@@ -356,9 +422,6 @@ typedef struct _PreferencesData PreferencesData;
 
 struct _PreferencesData
 {
-  /* General information.  */
-  GnomePropertyBox *property_box;
-  
   /* Page one.  */
   GtkWidget        *game_list;
   gint              selected_game;
@@ -368,71 +431,69 @@ struct _PreferencesData
   
   guint32           joystick_deviceid;
   gfloat            joystick_switch_level;
+
+
+  /* Page three */
+
+  /* Page four */
+  gchar            *scroll_method_name;
+
 };
 
 
 static void
-preferences_apply_cb (GtkWidget *w, gint page, gpointer data)
+preferences_response_cb (GtkWidget *w, gint response_id, gpointer data)
 {
   PreferencesData *prdata= (PreferencesData *) data;
 
   g_return_if_fail (prdata != NULL);
 
-  switch (page)
-    {
-    case 0:
-      /* FIXME: Add some checks and warnings here.  */
-      if (prdata->selected_game > -1)
-	load_game_by_number (prdata->selected_game);
-      break;
+  switch(response_id)
+  {
+  case GTK_RESPONSE_DELETE_EVENT:
+  case GTK_RESPONSE_ACCEPT:
+       /* FIXME: Add some checks and warnings here.  */
+       if (prdata->selected_game > -1)
+            load_game_by_number (prdata->selected_game);
+       
+       joystick_set_properties (prdata->joystick_deviceid,
+                                prdata->joystick_switch_level);
+       set_scroll_method (prdata->scroll_method_name);
+       /* After setting all needed values, we can save the programs
+          state to disc.  */
+       preferences_save_global ();
+       gtk_widget_destroy (preferences_dialog);
+       preferences_dialog = NULL;
+       break;
+
+  case GTK_RESPONSE_HELP:
+       break;
+  }
 
-    case 1:
-      joystick_set_properties (prdata->joystick_deviceid,
-			       prdata->joystick_switch_level);
-      break;
-    default:
-      /* After setting all needed values, we can save the programs
-         state to disc.  */
-      preferences_save_global ();
-      break;
-    }
 }
 
-
-static gint
-preferences_destroy_cb (GtkWidget *w, gpointer data)
-{
-  g_free (data);
-
-  preferences_dialog= NULL;
-
-  return FALSE;
-}
-
-
-static void
-preferences_changed_cb (GtkWidget *w, gpointer data)
-{
-  PreferencesData *prdata= (PreferencesData *) data;
-  
-  g_return_if_fail (prdata != NULL);
-
-  gnome_property_box_changed (prdata->property_box);
-}
-
-
 static void 
-game_selector_select_row (GtkCList * clist,
-			  gint row, gint column,
-			  GdkEvent * event, gpointer data)
+game_selector_select_row (GtkTreeSelection * selection,
+                          gpointer data)
 {
   PreferencesData *prdata= (PreferencesData *) data;
-  
+  GtkTreeView * view;
+  GtkTreeIter iter;
+  GtkTreeModel *model;
+
   g_return_if_fail (prdata != NULL);
 
-  preferences_changed_cb (GTK_WIDGET (clist), data);
+  view = gtk_tree_selection_get_tree_view(selection);
+
+  if(gtk_tree_selection_get_selected(selection, &model, &iter))
+  {
+       int index;
+       gtk_tree_model_get(model, &iter, 
+                          INDEX_INT, &index,
+                          -1);
+       prdata->selected_game= index;
+  }
 
-  prdata->selected_game= row;
 }
 
 
@@ -456,7 +517,6 @@ preferences_set_joystick_device (GtkWidget *widget, gpointer data)
       gtk_widget_set_sensitive (prdata->level_frame, TRUE);
     }
 #endif
-  gnome_property_box_changed (prdata->property_box);
 }
 
 
@@ -468,99 +528,165 @@ preferences_set_joystick_switch_level (GtkAdjustment *adjust, gpointer data)
 
   prdata->joystick_switch_level= adjust->value;
 
-  gnome_property_box_changed (prdata->property_box);
 }
 
 
+
+/* The scroll method callbacks.  */
+
+static void
+preferences_set_scroll_method (GtkWidget *widget, gpointer data)
+{
+
+  PreferencesData *prdata  = 
+    (PreferencesData *) gtk_object_get_user_data (GTK_OBJECT (widget));
+
+  prdata->scroll_method_name= (gchar *)data;
+
+}
+
+
+
+
 static GtkWidget *
 preferences_dialog_new (void)
 {
-  GtkWidget *propbox;
+  GtkWidget *properties;
   GtkWidget *box;
+  GtkWidget *notebook;
+  GtkWidget *list_view;
   GtkWidget *label;
-  GtkWidget *list;
   GtkWidget *scrolled;
-
+  GtkListStore *list;
+  GtkTreeIter iter;
+  GtkTreeSelection *selection;
+  GtkCellRenderer *renderer;
+  GtkTreeViewColumn *column;
+  
   PreferencesData *prdata;
-
+  
   prdata= g_malloc (sizeof (PreferencesData));
-
-  propbox= gnome_property_box_new ();
-  prdata->property_box= GNOME_PROPERTY_BOX (propbox);
   
-  gtk_window_set_title (GTK_WINDOW(&GNOME_PROPERTY_BOX(propbox)->dialog.window),
-			_("Gnome-Stones Preferences"));
-  gtk_window_set_wmclass (GTK_WINDOW(&GNOME_PROPERTY_BOX(propbox)->dialog.window),
-			  "gnome-stones", "preferences");
+  properties = gtk_dialog_new_with_buttons(_("GNOME-Stones Preferences"),
+                                           GTK_WINDOW(app),
+                                           /*GTK_DIALOG_MODAL |*/ GTK_DIALOG_DESTROY_WITH_PARENT,
+                                           GTK_STOCK_HELP, GTK_RESPONSE_HELP,
+                                           GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT,
+                                           NULL);
+  
+  notebook = gtk_notebook_new();
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG(properties)->vbox), notebook, 
+                      TRUE, TRUE, 0);
+  gtk_widget_show (notebook);
   
-
   /* The first page of our preferences dialog. */
   box= gtk_vbox_new (FALSE, GNOME_PAD);
   gtk_container_set_border_width (GTK_CONTAINER (box), GNOME_PAD_SMALL);
 
   /* The list of game names.  */
-  list= gtk_clist_new (3);
-  prdata->game_list= list;
+  list= gtk_list_store_new (N_COLS, 
+                            G_TYPE_STRING, /* Game title */ 
+                            G_TYPE_INT, /* Number of caves */
+                            G_TYPE_STRING, /* Filename */
+                            G_TYPE_INT); /* Index - remains hidden */
 
-  gtk_clist_set_column_title (GTK_CLIST (list), 0, _("Game title"));
-  gtk_clist_set_column_width (GTK_CLIST (list), 0, 250);
-  gtk_clist_set_column_title (GTK_CLIST (list), 1, _("Caves"));
-  gtk_clist_set_column_width (GTK_CLIST (list), 1, 50);
-  gtk_clist_set_column_justification (GTK_CLIST (list), 1, GTK_JUSTIFY_RIGHT);
-  gtk_clist_set_column_title (GTK_CLIST (list), 2, _("Filename"));
-  gtk_clist_column_titles_passive (GTK_CLIST (list));
-  gtk_clist_column_titles_show (GTK_CLIST (list));
-  gtk_widget_set_usize (list, -2, 200);
+  /* Populate list */
+  {
+    GList *tmp= games;
+    int index = 0;
+
+    while (tmp)
+      {
+	char buffer[10];
+	GameFile *file= (GameFile *)tmp->data;
+	gint      row;
+	
+        gtk_list_store_append(list, &iter);
+        gtk_list_store_set (list, &iter,
+                            GAME_STRING, file->gametitle,
+                            CAVES_INT, file->caves,
+                            FILENAME_STRING, file->filename,
+                            INDEX_INT, index,
+                            -1);
+        index++;
+	tmp= tmp->next;
+      }
+  }
+
+  /* Create view */
+  list_view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(list));
+
+  g_object_unref(list);
+
+  prdata->game_list= list_view;
+
+  renderer = gtk_cell_renderer_text_new();
+  column = gtk_tree_view_column_new_with_attributes(_("Game title"), 
+                                                 renderer, 
+                                                 "text", 0, 
+                                                 NULL);
+  gtk_tree_view_append_column(GTK_TREE_VIEW(list_view), column);
   
-  gtk_clist_set_selection_mode (GTK_CLIST (list), GTK_SELECTION_SINGLE);
+  renderer = gtk_cell_renderer_text_new();
+  column = gtk_tree_view_column_new_with_attributes(_("Caves"), renderer, 
+                                                 "text", 1, 
+                                                 NULL);
+  gtk_tree_view_append_column(GTK_TREE_VIEW(list_view), column);
+
+  renderer = gtk_cell_renderer_text_new();
+  column = gtk_tree_view_column_new_with_attributes(_("Filename"), renderer, 
+                                                 "text", 2, 
+                                                 NULL);
+  gtk_tree_view_append_column(GTK_TREE_VIEW(list_view), column);
 
   scrolled = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
                                   GTK_POLICY_ALWAYS,
                                   GTK_POLICY_AUTOMATIC);
-  gtk_container_add (GTK_CONTAINER (scrolled), list);
-  gtk_box_pack_start (GTK_BOX (box), scrolled, FALSE, FALSE, GNOME_PAD_SMALL);
 
-  gtk_clist_freeze (GTK_CLIST (list));
-  prdata->selected_game= -1;
+  gtk_container_add (GTK_CONTAINER (scrolled), list_view);
+  gtk_box_pack_start (GTK_BOX (box), scrolled, TRUE, TRUE, GNOME_PAD_SMALL);
+
+  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(list_view));
+  gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
+
   {
-    GList *tmp= games;
-    
-    while (tmp)
-      {
-	char buffer[10];
-	GameFile *file= (GameFile *)tmp->data;
-	char     *entry[3];
-	gint      row;
-	
-	entry[0]= file->gametitle;
-	sprintf (buffer, "%d", file->caves);
-	entry[1]= buffer;
-	entry[2]= g_basename (file->filename);
-	
-	row= gtk_clist_append (GTK_CLIST (list), entry);
-
-	if (game && strcmp (file->filename, game->filename) == 0)
-	  {
-	    gtk_clist_select_row (GTK_CLIST (list), row, 0);
-	    prdata->selected_game= row;
-	  }
-
-	tmp= tmp->next;
-      }
+       gboolean valid;
+       GtkTreeModel *model;
+       /* Get an iter associated with the view */
+  
+       model = gtk_tree_view_get_model(GTK_TREE_VIEW(list_view));
+       valid = gtk_tree_model_get_iter_first( model, &iter);
+  
+       while(valid)
+       {
+            /* Walk through list, testing each row */
+            gchar *filename;
+            int index;
+            gtk_tree_model_get(model, &iter,
+                               FILENAME_STRING, &filename,
+                               INDEX_INT, &index,
+                               -1 );
+            if (game && strcmp (filename, game->filename) == 0)
+            {
+                 gtk_tree_selection_select_iter (selection, &iter);
+                 prdata->selected_game= index;
+            }
+            valid = gtk_tree_model_iter_next (model, &iter);
+       }
   }
-  gtk_clist_thaw (GTK_CLIST (list));
-  gtk_signal_connect (GTK_OBJECT (list), "select_row", 
-		      GTK_SIGNAL_FUNC (game_selector_select_row),
-		      prdata);
 
-  gtk_widget_show (list);
+  g_signal_connect ( selection, 
+                     "changed", 
+                     GTK_SIGNAL_FUNC (game_selector_select_row),
+                     prdata);
+
+  gtk_widget_show (list_view);
   gtk_widget_show (scrolled);
   gtk_widget_show (box);
 
   label= gtk_label_new (_("Game"));
-  gtk_notebook_append_page (GTK_NOTEBOOK 
-			    (GNOME_PROPERTY_BOX (propbox)->notebook), 
+  gtk_notebook_append_page (GTK_NOTEBOOK (notebook), 
 			    box, label);
 
   /* The second page of our preferences dialog. */
@@ -605,11 +731,11 @@ preferences_dialog_new (void)
     menuitem= gtk_menu_item_new_with_label (_("disabled"));
     gtk_object_set_user_data (GTK_OBJECT (menuitem), prdata);
 #if 0
-    gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
+    g_signal_connect (GTK_OBJECT (menuitem), "activate",
 			(GtkSignalFunc) preferences_set_joystick_device,
 			GUINT_TO_POINTER (GDK_CORE_POINTER));
 #endif
-    gtk_menu_append (GTK_MENU (device_menu), menuitem);
+    gtk_menu_shell_append (GTK_MENU_SHELL (device_menu), menuitem);
     gtk_widget_show (menuitem);
 
 #if 0    
@@ -623,11 +749,11 @@ preferences_dialog_new (void)
 	    menuitem= gtk_menu_item_new_with_label (info->name);
 
 	    gtk_object_set_user_data (GTK_OBJECT (menuitem), prdata);
-            gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
+            g_signal_connect (GTK_OBJECT (menuitem), "activate",
                                 (GtkSignalFunc) preferences_set_joystick_device,
                                 GUINT_TO_POINTER (info->deviceid));
 
-	    gtk_menu_append (GTK_MENU (device_menu), menuitem);
+	    gtk_menu_shell_append (GTK_MENU_SHELL (device_menu), menuitem);
 	    gtk_widget_show (menuitem);
 	  }
 
@@ -663,7 +789,7 @@ preferences_dialog_new (void)
     adjust= gtk_adjustment_new (prdata->joystick_switch_level,
 				0.0, 1.0, 0.02, 0.1, 0.0);
     gtk_object_set_user_data (adjust, prdata);
-    gtk_signal_connect (adjust, "value_changed",
+    g_signal_connect (adjust, "value_changed",
 			(GtkSignalFunc) preferences_set_joystick_switch_level,
 			NULL);
     
@@ -683,33 +809,133 @@ preferences_dialog_new (void)
   gtk_widget_show (box);
 
   label= gtk_label_new (_("Joystick"));
-  gtk_notebook_append_page (GTK_NOTEBOOK 
-			    (GNOME_PROPERTY_BOX (propbox)->notebook), 
+  gtk_notebook_append_page (GTK_NOTEBOOK (notebook), 
 			    box, label);
 
-  /* The third page of our preferences dialog. 
-   *  box= gtk_vbox_new (FALSE, GNOME_PAD);
-   *  gtk_container_set_border_width (GTK_CONTAINER (box), GNOME_PAD_SMALL);
-   */
+  /* The third page of our preferences dialog. */
 
-  /*  label= gtk_label_new (_("Not yet implemented!"));
-   *  gtk_box_pack_start (GTK_BOX (box), label, TRUE, FALSE, GNOME_PAD_SMALL);
-   */
+  box= gtk_vbox_new (FALSE, GNOME_PAD);
+  gtk_container_set_border_width (GTK_CONTAINER (box), GNOME_PAD_SMALL);
+  
 
-   /*  gtk_widget_show (label); */
-   /*  gtk_widget_show (box); */
+  label= gtk_label_new (_("Sound enable/disable. Not yet implemented!"));
+  gtk_box_pack_start (GTK_BOX (box), label, TRUE, FALSE, GNOME_PAD_SMALL);
+  
 
-  /* label= gtk_label_new (_("Sound"));
-   *  gtk_notebook_append_page (GTK_NOTEBOOK
-   *                       (GNOME_PROPERTY_BOX (propbox)->notebook),
-   *                       box, label);
-   */
+  gtk_widget_show (label); 
+  gtk_widget_show (box); 
 
-  gtk_signal_connect (GTK_OBJECT (propbox), "destroy",
-		      GTK_SIGNAL_FUNC (preferences_destroy_cb), prdata);
-  gtk_signal_connect (GTK_OBJECT (propbox), "apply",
-		      GTK_SIGNAL_FUNC (preferences_apply_cb), prdata);
-  return propbox;
+  label= gtk_label_new (_("Sound"));
+  gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
+			    box, label);
+
+
+  /* Fourth page is miscellaneous stuff */
+  
+  box= gtk_vbox_new (FALSE, GNOME_PAD);
+  gtk_container_set_border_width (GTK_CONTAINER (box), GNOME_PAD_SMALL);
+  
+
+  
+  {
+    gint      i=0;
+    GtkWidget *frame;
+    GtkWidget *hbox;
+    GtkWidget *vbox;
+    GtkWidget *menuitem;
+    GtkWidget *optionmenu;
+    GtkWidget *scroll_method_menu;
+
+
+    frame= gtk_frame_new (_("Scroll method"));
+    gtk_box_pack_start (GTK_BOX (box), frame, FALSE, FALSE, GNOME_PAD_SMALL);
+    gtk_widget_show (frame);
+
+    vbox= gtk_vbox_new (FALSE, GNOME_PAD);
+    gtk_container_add (GTK_CONTAINER (frame), vbox);
+    gtk_widget_show (vbox);
+
+    hbox= gtk_hbox_new (FALSE, GNOME_PAD);
+    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, GNOME_PAD_SMALL);
+    gtk_widget_show (hbox);
+
+    /*
+    label= gtk_label_new (_("Scroll method:"));
+    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, GNOME_PAD_SMALL);
+    gtk_widget_show (label);
+    */
+    
+    scroll_method_menu= gtk_menu_new ();
+
+
+    prdata->scroll_method_name = scroll_method_name();
+
+    menuitem= gtk_menu_item_new_with_label (_("Atari like scrolling"));
+    gtk_object_set_user_data (GTK_OBJECT (menuitem), prdata);
+    g_signal_connect (GTK_OBJECT (menuitem), "activate",
+			(GtkSignalFunc) preferences_set_scroll_method,
+			"atari_scroll");
+    gtk_menu_shell_append (GTK_MENU_SHELL (scroll_method_menu), menuitem);
+    gtk_widget_show (menuitem);
+
+    if (!strcmp(prdata->scroll_method_name,"atari_scroll"))
+      gtk_menu_set_active (GTK_MENU (scroll_method_menu), i++);
+    ++i;
+
+
+
+
+    menuitem= gtk_menu_item_new_with_label (_("Smooth scrolling"));
+    gtk_object_set_user_data (GTK_OBJECT (menuitem), prdata);
+    g_signal_connect (GTK_OBJECT (menuitem), "activate",
+			(GtkSignalFunc) preferences_set_scroll_method,
+			"smooth_scroll");
+    gtk_menu_shell_append (GTK_MENU_SHELL (scroll_method_menu), menuitem);
+    gtk_widget_show (menuitem);
+
+    if (!strcmp(prdata->scroll_method_name,"smooth_scroll"))
+      gtk_menu_set_active (GTK_MENU (scroll_method_menu), i++);
+    ++i;
+
+
+
+    menuitem= gtk_menu_item_new_with_label (_("Always in the center"));
+    gtk_object_set_user_data (GTK_OBJECT (menuitem), prdata);
+    g_signal_connect (GTK_OBJECT (menuitem), "activate",
+			(GtkSignalFunc) preferences_set_scroll_method,
+			"center_scroll");
+    gtk_menu_shell_append (GTK_MENU_SHELL (scroll_method_menu), menuitem);
+    gtk_widget_show (menuitem);
+
+    if (!strcmp(prdata->scroll_method_name,"center_scroll"))
+      gtk_menu_set_active (GTK_MENU (scroll_method_menu), i);
+    ++i;
+
+    
+
+   
+    optionmenu= gtk_option_menu_new ();
+    gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu), scroll_method_menu);
+    gtk_box_pack_start (GTK_BOX (hbox), optionmenu, FALSE, FALSE, 2);
+    gtk_widget_show (optionmenu);
+    
+    gtk_widget_show (label);
+    gtk_widget_show (hbox);
+    gtk_widget_show (optionmenu);
+
+  }
+
+  gtk_widget_show (box); 
+
+  label= gtk_label_new (_("Misc."));
+  gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
+			    box, label);
+
+
+  g_signal_connect (GTK_OBJECT (properties), "response",
+		      GTK_SIGNAL_FUNC (preferences_response_cb), prdata);
+
+  return properties;
 }
 
 
@@ -718,10 +944,10 @@ preferences_dialog_show (void)
 {
   if (!preferences_dialog)
     {
-      preferences_dialog= preferences_dialog_new ();
+      preferences_dialog = preferences_dialog_new ();
     }
-  
-  gtk_widget_show (preferences_dialog);
+
+  gtk_window_present (GTK_WINDOW (preferences_dialog));
 }
 
 
@@ -730,23 +956,16 @@ preferences_dialog_show (void)
 /* Initialize the session management stuff.  */
 
 
-/* FIXME: should move to main.c.  */
-void
-gstones_exit (GnomeClient *client, gpointer client_data)
-{
-  exit (0);
-}
-
 
 void
 session_management_init (void)
 {
   GnomeClient *client= gnome_master_client ();
   
-  gtk_signal_connect (GTK_OBJECT (client), "save_yourself",
+  g_signal_connect (GTK_OBJECT (client), "save_yourself",
 		      GTK_SIGNAL_FUNC (preferences_save_local), 
 		      GINT_TO_POINTER (FALSE));
-  gtk_signal_connect (GTK_OBJECT (client), "die",
+  g_signal_connect (GTK_OBJECT (client), "die",
 		      GTK_SIGNAL_FUNC (gstones_exit), NULL);
 }
 
@@ -759,3 +978,8 @@ session_management_init (void)
 /* eval:(add-hook 'write-file-hooks 'time-stamp) */
 /* eval:(setq time-stamp-format '(time-stamp-yyyy/mm/dd time-stamp-hh:mm:ss user-login-name)) */
 /* End: */
+
+
+
+
+
