@@ -70,7 +70,8 @@ Tetris::Tetris(int cmdlLevel):
 	bgimage(0),
 	setupdialog(0), 
 	cmdlineLevel(cmdlLevel), 
-	fastFall(false)
+	fastFall(false),
+        dropBlock(false)
 {
 	pic = new GdkPixbuf*[tableSize];
 	for (int i = 0; i < tableSize; ++i)
@@ -121,7 +122,7 @@ Tetris::Tetris(int cmdlLevel):
 
 	/* init gconf */
 	gconf_client = gconf_client_get_default ();
-	if (!games_gconf_sanity_check_string (gconf_client, "")) {
+	if (!games_gconf_sanity_check_string (gconf_client, "/apps/gnometris/options/block_pixmap")) {
 		exit(1);
 	}
 	gconf_client_add_dir (gconf_client_get_default (), "/apps/gnometris/options", GCONF_CLIENT_PRELOAD_NONE, NULL);
@@ -174,6 +175,31 @@ Tetris::Tetris(int cmdlLevel):
 	gtk_widget_set_sensitive(gameMenuPtr[1].widget, FALSE);
 	gtk_widget_set_sensitive(gameMenuPtr[4].widget, FALSE);
 	gtk_widget_set_sensitive(gameSettingsPtr[0].widget, TRUE);
+
+        pauseMessage = gnome_canvas_item_new (gnome_canvas_root(GNOME_CANVAS(field->getWidget())),
+                                              gnome_canvas_text_get_type(),
+                                              "fill_color",
+                                              "white",
+                                              "x", COLUMNS*BLOCK_SIZE/2.0,
+                                              "y", LINES*BLOCK_SIZE/2.0,
+                                              "text", _("Paused"),
+                                              "size_points", 36.0,
+                                              0
+                                              );
+        gnome_canvas_item_hide (pauseMessage);
+
+        gameoverMessage = gnome_canvas_item_new (gnome_canvas_root(GNOME_CANVAS(field->getWidget())),
+                                                 gnome_canvas_text_get_type(),
+                                                 "fill_color",
+                                                 "white",
+                                                 "x", COLUMNS*BLOCK_SIZE/2.0,
+                                                 "y", LINES*BLOCK_SIZE/2.0,
+                                                 "text", _("Game Over"),
+                                                 "size_points", 36.0,
+                                                 0
+                                                 );
+        gnome_canvas_item_hide (gameoverMessage);
+
 }
 
 Tetris::~Tetris()
@@ -217,6 +243,7 @@ void
 Tetris::setupPixmap()
 {
 	char *pixname, *fullpixname;
+        gchar * s;
 	
 	pixname = g_strdup_printf("gnometris/%s", blockPixmap);
 	fullpixname = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP, pixname, FALSE, NULL);
@@ -274,8 +301,20 @@ Tetris::setupPixmap()
 
 	if (g_file_test (fullpixname, G_FILE_TEST_EXISTS)) 
 		bgimage = gdk_pixbuf_new_from_file(fullpixname, NULL);
-	else
-		bgimage = 0;
+	else {
+                /* If we can't find a .png look for a .jpg, this is
+                 * especially pertinant since a lot of the original
+                 * backgrounds have been converted for size reasons. */
+                s = g_strrstr(fullpixname, ".png");
+                g_strlcpy(s, ".jpg", 5);
+                if (g_file_test (fullpixname, G_FILE_TEST_EXISTS)) {
+                        bgimage = gdk_pixbuf_new_from_file(fullpixname, NULL);
+                        s = g_strrstr (bgPixmap, ".png");
+                        g_strlcpy (s, ".jpg", 5);
+                        gconf_client_set_string (gconf_client, "/apps/gnometris/options/background_pixmap", bgPixmap, NULL);
+                } else
+                        bgimage = 0;
+        }
 
 	if (field)
 	{
@@ -484,7 +523,7 @@ Tetris::fillMenu(GtkWidget *menu, char *pixname, char *dirname,
 	{
 		s = g_strdup(e->d_name);
 
-		if (!strstr (e->d_name, ".png")) 
+		if (!(strstr (e->d_name, ".png") || strstr (e->d_name, ".jpg"))) 
 		{
 			free(s);
 			continue;
@@ -711,7 +750,7 @@ Tetris::gameEnd(GtkWidget *widget, void *d)
 {
 	Tetris *t = (Tetris*) d;
 	
-	gtk_timeout_remove(t->timeoutId);
+	g_source_remove(t->timeoutId);
 	t->timeoutId = -1;
 	blocknr_next = -1;
 	t->endOfGame();
@@ -733,13 +772,13 @@ Tetris::gameQuit(GtkWidget *widget, void *d)
 void
 Tetris::generateTimer(int level)
 {
-	gtk_timeout_remove(timeoutId);
+	g_source_remove(timeoutId);
 
 	int intv = 1000 - 100 * (level - 1);
 	if (intv <= 0)
 		intv = 100;
 		
-	timeoutId = gtk_timeout_add(intv, timeoutHandler, this);
+	timeoutId = g_timeout_add_full(0, intv, timeoutHandler, this, NULL);
 }
 
 void
@@ -777,10 +816,10 @@ Tetris::timeoutHandler(void *d)
 		if (res)
 		{
 			t->manageFallen();
-			if(t->fastFall)
+			if(t->fastFall) {
 				t->scoreFrame->incScore(t->fastFallPoints);
-			else
 				t->fastFall = false;
+			}
 		}
 		else if(t->fastFall)
 			++t->fastFallPoints;
@@ -793,7 +832,8 @@ gint
 Tetris::eventHandler(GtkWidget *widget, GdkEvent *event, void *d)
 {
 	Tetris *t = (Tetris*) d;
-	
+        int bonus;
+        
 	if (t->timeoutId == -1)
 		return FALSE;
 	
@@ -825,23 +865,36 @@ Tetris::eventHandler(GtkWidget *widget, GdkEvent *event, void *d)
 		{
 		case GDK_Left:
 			res = t->ops->moveBlockLeft();
+                        t->onePause = false;
 			break;
 		case GDK_Right:
 			res = t->ops->moveBlockRight();
+                        t->onePause = false;
 			break;
 		case GDK_Up:
 			res = t->ops->rotateBlock(rotateCounterClockWise);
+                        t->onePause = false;
 			break;
 		case GDK_Down:
-			if (!t->fastFall)
+			if (!t->fastFall && !t->onePause)
 			{
 				t->fastFall = true;
 				t->fastFallPoints = 0;
-				gtk_timeout_remove(t->timeoutId);
-				t->timeoutId = gtk_timeout_add(10, timeoutHandler, t);
+				g_source_remove(t->timeoutId);
+				t->timeoutId = g_timeout_add_full(0, 10, timeoutHandler, t, NULL);
 				res = true;
 			}
 			break;
+		case GDK_space:
+                        if (!t->dropBlock)
+                        {
+                                t->dropBlock = true;
+                                bonus = t->ops->dropBlock();
+                                t->scoreFrame->incScore(bonus);
+                                t->manageFallen();
+                                res = TRUE;
+                        }        
+			break;
 		default:
 			return FALSE;
 		}
@@ -862,8 +915,7 @@ Tetris::eventHandler(GtkWidget *widget, GdkEvent *event, void *d)
 			}
 			break;
 		case GDK_space:
-			t->ops->dropBlock();
-			res = TRUE;
+                        t->dropBlock = false;
 			break;
 		default:
 			return FALSE;
@@ -884,6 +936,11 @@ void
 Tetris::togglePause()
 {
 	paused = !paused;
+        if (paused) {
+                gnome_canvas_item_show (pauseMessage);
+                gnome_canvas_item_raise_to_top (pauseMessage);
+        } else
+                gnome_canvas_item_hide (pauseMessage);
 }
 
 void
@@ -897,7 +954,7 @@ Tetris::generate()
 	}
 	else
 	{
-		gtk_timeout_remove(timeoutId);
+		g_source_remove(timeoutId);
 		timeoutId = -1;
 		blocknr_next = -1;
 		
@@ -914,6 +971,8 @@ Tetris::endOfGame()
 	color_next = -1;
 	blocknr_next = -1;
 	rot_next = -1;
+        gnome_canvas_item_show (gameoverMessage);
+        gnome_canvas_item_raise_to_top (gameoverMessage);
 
 	preview->clear ();
 	
@@ -943,7 +1002,7 @@ Tetris::gameNew(GtkWidget *widget, void *d)
 
 	if (t->timeoutId) 
 	{
-		gtk_timeout_remove(t->timeoutId);
+		g_source_remove(t->timeoutId);
 		t->timeoutId = -1;
 
 		if (t->scoreFrame->getScore() > 0) 
@@ -957,7 +1016,7 @@ Tetris::gameNew(GtkWidget *widget, void *d)
 	t->scoreFrame->setLevel(level);
 	t->scoreFrame->setStartingLevel(level);
 
-	t->timeoutId = gtk_timeout_add(1000 - 100 * (level - 1), timeoutHandler, t);
+	t->timeoutId = g_timeout_add_full(0, 1000 - 100 * (level - 1), timeoutHandler, t, NULL);
 	t->ops->emptyField(t->line_fill_height,t->line_fill_prob);
 
 	t->scoreFrame->resetLines();
@@ -971,6 +1030,9 @@ Tetris::gameNew(GtkWidget *widget, void *d)
 	gtk_widget_set_sensitive(t->gameMenuPtr[4].widget, TRUE);
 	gtk_widget_set_sensitive(t->gameSettingsPtr[0].widget, FALSE);
 
+        gnome_canvas_item_hide (t->pauseMessage);
+        gnome_canvas_item_hide (t->gameoverMessage);
+        
 	return TRUE;
 }
 
