@@ -10,18 +10,21 @@
  *
  */
 
-#include <sys/types.h>
-#include <string.h>
 #include <stdlib.h>
 #include <time.h>
-#include <dirent.h>
 
 #include <config.h>
 #include <gnome.h>
 #include <libgnomeui/gnome-window-icon.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gconf/gconf-client.h>
+#include <games-gconf.h>
 
+/* Define a sensible alternative to ngettext if we don't have it. Note that
+ * this is only sensible in the context of same-gnome. */
+#ifndef HAVE_NGETTEXT
+#define ngettext(one,lots,n) gettext(lots)
+#endif
 
 #define STONE_SIZE 40
 #define STONE_COLS  15
@@ -32,7 +35,7 @@
 		     GDK_LEAVE_NOTIFY_MASK          |\
 		     GDK_POINTER_MOTION_MASK)
 
-static GtkWidget *pref_dialog, *scorew, *markedw;
+static GtkWidget *pref_dialog, *scorew;
 static GtkWidget *app, *draw_area, *vb, *appbar;
 static GdkPixbuf *image;
 static GdkPixmap *stones, *mask;
@@ -43,6 +46,7 @@ static int score;
 static gchar *scenario;
 static gint restarted;
 static gboolean game_over = FALSE;
+static gboolean ignore_selection;
 
 void update_score_state ();
 
@@ -150,7 +154,6 @@ move_tagged_balls (void *data)
 			field [x][y].frame = (field [x][y].frame + 1) % nstones;
 			draw_ball (x, y);
 		}
-	gdk_flush ();
 	return 1;
 }
 
@@ -158,7 +161,7 @@ static void
 disable_timeout ()
 {
 	if (ball_timeout_id != -1){
-		gtk_timeout_remove (ball_timeout_id);
+		g_source_remove (ball_timeout_id);
 		ball_timeout_id = -1;
 	}
 }
@@ -180,12 +183,12 @@ mark_balls (int x, int y)
 	
 	if (tagged_count > 1) {
 		char *b;
-		ball_timeout_id = gtk_timeout_add (100, move_tagged_balls, 0);
-		b = g_strdup_printf ("%.2d", tagged_count);
-		gtk_label_set_text (GTK_LABEL(markedw), b);
+		ball_timeout_id = g_timeout_add (100, move_tagged_balls, 0);
+		b = g_strdup_printf (ngettext("%d stone selected", "%d stones selected", tagged_count), tagged_count);
+                gnome_appbar_set_status (GNOME_APPBAR(appbar), b);
 		g_free (b);
 	} else
-		gtk_label_set_text (GTK_LABEL(markedw), "00");
+                gnome_appbar_set_status (GNOME_APPBAR(appbar), _("No stones selected"));
 }
 
 static void
@@ -262,7 +265,7 @@ show_scores (guint pos)
 {
 	GtkWidget *dialog;
 
-	dialog = gnome_scores_display (_("The Same Gnome"), "same-gnome", NULL, pos);
+	dialog = gnome_scores_display (_("The Same GNOME"), "same-gnome", NULL, pos);
 	if (dialog != NULL) {
 		gtk_window_set_transient_for (GTK_WINDOW(dialog), GTK_WINDOW(app));
 		gtk_window_set_modal (GTK_WINDOW(dialog), TRUE);
@@ -352,6 +355,7 @@ area_event (GtkWidget *widget, GdkEvent *event, void *d)
 		old_y = -1;
 		disable_timeout ();
 		untag_all ();
+                gnome_appbar_set_status (GNOME_APPBAR(appbar), "");
 		return TRUE;
 
 	default:
@@ -408,12 +412,17 @@ load_scenario (char *fname)
 			GTK_MESSAGE_ERROR,
 			GTK_BUTTONS_OK,
 			_("Could not find the theme:\n%s\n\n"
-			"Please check you Same Gnome instalation"), fn);
+			"Please check your Same GNOME installation."), fn);
 
 		gtk_dialog_set_default_response (GTK_DIALOG (box), GTK_RESPONSE_OK);
 		gtk_dialog_run (GTK_DIALOG (box));
 		gtk_widget_destroy (box);
-		exit (1);
+                /* We may not have called gtk_main yet, but if we do we want
+                 * to exit nicely. */
+                if (gtk_main_level() > 0)
+                  gtk_main_quit ();
+                else
+                  exit (1);
 	}
 
 	g_free (scenario);
@@ -431,13 +440,18 @@ load_scenario (char *fname)
 			GTK_DIALOG_DESTROY_WITH_PARENT,
 			GTK_MESSAGE_ERROR,
 			GTK_BUTTONS_OK,
-			_("Same Gnome can't load the image file:\n%s\n\n"
-			 "Please check your Same Gnome installation"), fn);
+			_("Same GNOME can't load the image file:\n%s\n\n"
+			 "Please check your Same GNOME installation"), fn);
 
 		gtk_dialog_set_default_response (GTK_DIALOG (box), GTK_RESPONSE_OK);
 		gtk_dialog_run (GTK_DIALOG (box));
 		gtk_widget_destroy (box);
-		gtk_main_quit ();
+                /* We may not have called gtk_main yet, but if we do we want
+                 * to exit nicely. */
+                if (gtk_main_level() > 0)
+                  gtk_main_quit ();
+                else
+                  exit(1);
 	}
 
 	gdk_pixbuf_render_pixmap_and_mask (image, &stones, &mask, 127);
@@ -495,46 +509,64 @@ game_new_callback (GtkWidget *widget, void *data)
 }
 
 static void
-set_selection (GtkWidget *widget, gpointer data)
+set_selection (GtkTreeSelection *selection, gpointer data)
 {
-	load_scenario (data);
-	gconf_client_set_string (conf_client, "/apps/same-gnome/tileset", data, NULL);
+        gchar * filename;
+        GtkTreeModel * model;
+        GtkTreeIter iter;
+
+        /* This is here because I can't figure out how to set the list
+         * so nothing is selected when it is created. */
+        if (ignore_selection) {
+                ignore_selection = FALSE;
+                return;
+        }
+
+        gtk_tree_selection_get_selected (selection, &model, &iter);
+        gtk_tree_model_get (model, &iter, 1, &filename, -1);
+
+	load_scenario (filename);
+	gconf_client_set_string (conf_client, "/apps/same-gnome/tileset", filename, NULL);
 }
 
 static void
-fill_menu (GtkWidget *menu)
+fill_list (GtkListStore *list)
 {
-	struct dirent *e;
-	DIR *dir;
-        int itemno = 0;
+        GDir * dir;
+        gchar * filename;
+        GtkTreeIter iter;
 	
-	dir = opendir (PIXMAPDIR);
+	dir = g_dir_open (PIXMAPDIR, 0, NULL);
 
 	if (!dir)
 		return;
 	
-	while ((e = readdir (dir)) != NULL){
-		GtkWidget *item;
-		char *name, *label, *p;
-		name = strdup (e->d_name);
+	while ((filename = g_strdup (g_dir_read_name (dir))) != NULL){
+		gchar *name;
+                gchar *suffix;
+                
+                if (!g_file_test (filename, G_FILE_TEST_IS_REGULAR)) {
+                        g_free(filename);
+                        continue;
+                }
 
-		if (!(p = strstr (name, ".png"))) {
-			free (name);
-			continue;
-		}
-		
-		item = gtk_menu_item_new_with_label (g_strndup (name, p - name ));
-		gtk_widget_show (item);
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-		g_signal_connect (G_OBJECT (item), "activate",
-				  G_CALLBACK (set_selection), name);
-	  
-	        if (!strcmp(scenario, name))
-			gtk_menu_set_active(GTK_MENU(menu), itemno);
-	  
-	        itemno++;
+                /* We strip any trailing suffix, any -sync suffix
+                 * and convert '_' to ' '. This is brutal code. */
+                name = g_strdup(filename);
+                suffix = g_strrstr(name,".");
+                if (suffix) *suffix = '\0';
+                suffix = g_strrstr(name,"-sync");
+                if (suffix) *suffix = '\0';
+                suffix = name;
+                while (*suffix) {
+                        if (*suffix == '_') *suffix = ' ';
+                        suffix++;
+                }
+                
+                gtk_list_store_append (list, &iter);
+                gtk_list_store_set (list, &iter, 0, name, 1, filename, -1);
 	}
-	closedir (dir);
+	g_dir_close (dir);
 }
 
 static void
@@ -547,15 +579,20 @@ pref_dialog_response (GtkDialog *dialog, gint response, gpointer data)
 static void
 game_preferences_callback (GtkWidget *widget, void *data)
 {
-	GtkWidget *menu, *omenu, *l, *hb;
-	GtkWidget *button;
+	GtkWidget *listview, *frame;
+	GtkWidget *button, *scroll;
+        GtkTreeViewColumn *column;
+        GtkTreeSelection * select;
+        GtkListStore *list;
 
 	if (pref_dialog) {
 		gtk_window_present (GTK_WINDOW (pref_dialog));
 		return;
 	}
-	
-	pref_dialog = gtk_dialog_new_with_buttons (_("Preferences"),
+
+        ignore_selection = TRUE;
+        
+	pref_dialog = gtk_dialog_new_with_buttons (_("Same GNOME Preferences"),
 			GTK_WINDOW (app),
 			GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR,
 			GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
@@ -566,25 +603,42 @@ game_preferences_callback (GtkWidget *widget, void *data)
 	g_signal_connect (G_OBJECT (pref_dialog), "response",
 			  G_CALLBACK(pref_dialog_response), NULL);
 
-	omenu = gtk_option_menu_new ();
-	menu = gtk_menu_new ();
-	fill_menu (menu);
-	gtk_widget_show (omenu);
-	gtk_option_menu_set_menu (GTK_OPTION_MENU(omenu), menu);
+        list = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
+	fill_list (list);
 
-	hb = gtk_hbox_new (FALSE, FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER (hb), 4);
-	gtk_widget_show (hb);
-	
-	l = gtk_label_new (_("Select scenario:"));
-	gtk_widget_show (l);
-	    
-	gtk_box_pack_start (GTK_BOX(hb), l, TRUE, FALSE, 4);
-	gtk_box_pack_start_defaults (GTK_BOX(hb), omenu);
+        listview = gtk_tree_view_new_with_model (GTK_TREE_MODEL (list));
+        column = gtk_tree_view_column_new_with_attributes (_("Theme"),
+                                                           gtk_cell_renderer_text_new (),
+                                                           "text", 0,
+                                                           NULL);
+        gtk_tree_view_append_column (GTK_TREE_VIEW (listview),
+                                     GTK_TREE_VIEW_COLUMN (column));
+        gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (listview), FALSE);
+        
+        select = gtk_tree_view_get_selection (GTK_TREE_VIEW (listview));
+        gtk_tree_selection_set_mode (select, GTK_SELECTION_BROWSE);
+        gtk_tree_selection_unselect_all (select);
+        g_signal_connect (G_OBJECT (select), "changed",
+                          G_CALLBACK (set_selection), NULL);
+        
+        scroll = gtk_scrolled_window_new (NULL, NULL);
+        gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll),
+                                        GTK_POLICY_AUTOMATIC,
+                                        GTK_POLICY_AUTOMATIC);
+        gtk_widget_set_size_request (scroll, 250, 200);
+        gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scroll),
+                                             GTK_SHADOW_ETCHED_IN);
+        gtk_container_add (GTK_CONTAINER(scroll), listview);
+        gtk_widget_set (scroll, "border-width", GNOME_PAD, NULL);
+        
+        frame = gtk_frame_new (_("Theme"));
+        gtk_container_add (GTK_CONTAINER (frame), scroll);
+        gtk_widget_set (frame, "border-width", GNOME_PAD_SMALL, NULL);
 
-	gtk_box_pack_start_defaults (GTK_BOX(GTK_DIALOG(pref_dialog)->vbox), hb);
-	
-        gtk_widget_show (pref_dialog);
+        gtk_box_pack_start (GTK_BOX (GTK_DIALOG(pref_dialog)->vbox),
+                            frame, TRUE, TRUE, GNOME_PAD_SMALL);
+
+        gtk_widget_show_all (pref_dialog);
 }
 
 static int
@@ -622,13 +676,16 @@ game_about_callback (GtkWidget *widget, void *data)
 		}
 	}
 	
-	about = gnome_about_new (_("The Same Gnome"), VERSION,
+	about = gnome_about_new (_("The Same GNOME"), VERSION,
 				 "(C) 1997-1998 the Free Software Foundation",
 				 _("Original idea from KDE's same game program."),
 				 (const char **)authors,
 				 (const char **)documenters,
 				 strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
                                  pixbuf);
+	if (pixbuf != NULL)
+		gdk_pixbuf_unref (pixbuf);
+	
 	gtk_window_set_transient_for (GTK_WINDOW (about), GTK_WINDOW (app));
 	gtk_widget_show (about);
 
@@ -641,30 +698,6 @@ game_about_callback (GtkWidget *widget, void *data)
 static int
 game_quit_callback (GtkWidget *widget, void *data)
 {
-	if (!game_over) {
-		GtkWidget *box;
-		gint response;
-		
-		box = gtk_message_dialog_new (GTK_WINDOW (app),
-				GTK_DIALOG_DESTROY_WITH_PARENT,
-				GTK_MESSAGE_QUESTION,
-				GTK_BUTTONS_NONE,
-				_("Are you sure you want to quit Same Gnome?"));
-
-		gtk_dialog_add_buttons (GTK_DIALOG (box),
-						GTK_STOCK_CANCEL,
-						GTK_RESPONSE_CANCEL,
-						GTK_STOCK_QUIT,
-						GTK_RESPONSE_YES,
-						NULL);
-
-		gtk_dialog_set_default_response (GTK_DIALOG (box), GTK_RESPONSE_YES);
-		response = gtk_dialog_run (GTK_DIALOG (box));
-		gtk_widget_destroy (box);
-
-		if (response != GTK_RESPONSE_YES)
-			return TRUE;
-	}
 	gtk_main_quit ();
 	return FALSE;
 }
@@ -806,7 +839,7 @@ main (int argc, char *argv [])
 		{ "scenario", 's', POPT_ARG_STRING, &fname, 0, N_("Set game scenario"), N_("NAME") },
 		{ NULL, '\0', 0, NULL, 0 }
 	};
-	GtkWidget *label, *separator;
+        GtkWidget * label;
 	GnomeClient *client;
 
 	gnome_score_init("same-gnome");
@@ -836,10 +869,13 @@ main (int argc, char *argv [])
 
 	/* Get the default GConfClient */
 	conf_client = gconf_client_get_default ();
+	if (!games_gconf_sanity_check_string (conf_client, "/apps/same-gnome/tileset")) {
+		return 1;
+	}
 
 	srand (time (NULL));
 
-	app = gnome_app_new("same-gnome", _("Same Gnome"));
+	app = gnome_app_new("same-gnome", _("Same GNOME"));
 
         gtk_window_set_policy(GTK_WINDOW(app), FALSE, FALSE, TRUE);
 	g_signal_connect (G_OBJECT (app), "delete_event",
@@ -849,7 +885,7 @@ main (int argc, char *argv [])
 	gnome_app_set_statusbar(GNOME_APP (app), GTK_WIDGET(appbar));
 
 	gnome_appbar_set_status(GNOME_APPBAR (appbar),
-				_("Welcome to Same Gnome!"));
+				_("Welcome to Same GNOME!"));
 
 	gnome_app_create_menus(GNOME_APP(app), mainmenu);
 
@@ -866,14 +902,6 @@ main (int argc, char *argv [])
 	create_same_board (fname);
 
 	update_score_state ();
-
-	separator = gtk_vseparator_new ();
-	label = gtk_label_new (_("Marked: "));
-	markedw = gtk_label_new ("00");
-
-	gtk_box_pack_start(GTK_BOX(appbar), label, FALSE, TRUE, 0);
-	gtk_box_pack_start(GTK_BOX(appbar), markedw, FALSE, TRUE, 0);
-	gtk_box_pack_start(GTK_BOX(appbar), separator, FALSE, TRUE, 5);
 	
 	label = gtk_label_new (_("Score: "));
 	scorew = gtk_label_new ("");
